@@ -24,6 +24,10 @@
 
 #include <qmessagebox.h>
 #include <qlayout.h>
+#include <qradiobutton.h>
+#include <qstringlist.h>
+#include <qregexp.h>
+#include <qcheckbox.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -37,14 +41,17 @@
 #include <kpopupmenu.h>
 #include <kiconloader.h>
 #include <kguiitem.h>
+#include <klineedit.h>
+#include <keditlistbox.h>
 
 // ----------------------------------------------------------------------------
 // Library Includes
 
 #include <aqbanking/imexporter.h>
 #include <aqbanking/jobgettransactions.h>
+#include <aqbanking/jobgetbalance.h>
 #include <aqbanking/job.h>
-#include <qbanking/qbpickstartdate.h>
+// #include <qbanking/qbpickstartdate.h>
 #include <qbanking/qbgui.h>
 #include <gwenhywfar/logger.h>
 #include <gwenhywfar/debug.h>
@@ -56,15 +63,19 @@
 #include "mymoneybanking.h"
 #include "kbjobview.h"
 #include "kbsettings.h"
+#include "kbaccountsettings.h"
 #include <kmymoney/mymoneyfile.h>
 #include <kmymoney/kmymoneyview.h>
-
+#include <kmymoney/mymoneykeyvaluecontainer.h>
+#include <kbpickstartdate.h>
 
 K_EXPORT_COMPONENT_FACTORY( kmm_kbanking,
                             KGenericFactory<KBankingPlugin>( "kmm_kbanking" ) )
 
 KBankingPlugin::KBankingPlugin(QObject *parent, const char* name, const QStringList&) :
-  KMyMoneyPlugin::OnlinePlugin(parent, name )
+  KMyMoneyPlugin::Plugin(parent, name),
+  KMyMoneyPlugin::OnlinePlugin(),
+  m_accountSettings(0)
 {
   m_kbanking=new KMyMoneyBanking(this, "KMyMoney");
 
@@ -75,6 +86,7 @@ KBankingPlugin::KBankingPlugin(QObject *parent, const char* name, const QStringL
     GWEN_Gui_SetGui(gui->getCInterface());
     GWEN_Logger_SetLevel(0, GWEN_LoggerLevel_Info);
     GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Debug);
+    m_kbanking->setGui(gui);
     if (m_kbanking->init() == 0) {
       // Tell the host application to load my GUI component
       setInstance(KGenericFactory<KBankingPlugin>::instance());
@@ -85,6 +97,9 @@ KBankingPlugin::KBankingPlugin(QObject *parent, const char* name, const QStringL
 
       // create actions
       createActions();
+
+      // load protocol conversion list
+      loadProtocolConversion();
     }
     else {
       kdWarning() << "Could not initialize KBanking online banking interface" << endl;
@@ -105,16 +120,73 @@ KBankingPlugin::~KBankingPlugin()
 }
 
 
+void KBankingPlugin::loadProtocolConversion(void)
+{
+  m_protocolConversionMap.clear();
+  m_protocolConversionMap["aqhbci"] = "HBCI";
+  m_protocolConversionMap["aqofxconnect"] = "OFX";
+  m_protocolConversionMap["aqyellownet"] = "YellowNet";
+  m_protocolConversionMap["aqgeldkarte"] = "Geldkarte";
+  m_protocolConversionMap["aqdtaus"] = "DTAUS";
+}
 
 void KBankingPlugin::protocols(QStringList& protocolList) const
 {
   std::list<std::string> list = m_kbanking->getActiveProviders();
   std::list<std::string>::iterator it;
-  for(it = list.begin(); it != list.end(); ++it)
-    protocolList << (*it);
+  for(it = list.begin(); it != list.end(); ++it) {
+    // skip the dummy
+    if(*it == "aqnone")
+      continue;
+    QMap<QString, QString>::const_iterator it_m;
+    it_m = m_protocolConversionMap.find(*it);
+    if(it_m != m_protocolConversionMap.end())
+      protocolList << (*it_m);
+    else
+      protocolList << (*it);
+  }
 }
 
+QWidget* KBankingPlugin::accountConfigTab(const MyMoneyAccount& acc, QString& name)
+{
+  const MyMoneyKeyValueContainer& kvp = acc.onlineBankingSettings();
+  name = i18n("Online settings");
+  m_accountSettings = new KBAccountSettings(acc, 0, 0);
+  m_accountSettings->m_usePayeeAsIsButton->setChecked(true);
+  m_accountSettings->m_transactionDownload->setChecked(kvp.value("kbanking-txn-download")!="no");
+  if(!kvp.value("kbanking-payee-regexp").isEmpty()) {
+    m_accountSettings->m_extractPayeeButton->setChecked(true);
+    m_accountSettings->m_payeeRegExpEdit->setText(kvp.value("kbanking-payee-regexp"));
+    m_accountSettings->m_memoRegExpEdit->setText(kvp.value("kbanking-memo-regexp"));
+    m_accountSettings->m_payeeExceptions->clear();
+    m_accountSettings->m_payeeExceptions->insertStringList(QStringList::split(";", kvp.value("kbanking-payee-exceptions")));
+  }
+  return m_accountSettings;
+}
 
+MyMoneyKeyValueContainer KBankingPlugin::onlineBankingSettings(const MyMoneyKeyValueContainer& current)
+{
+  MyMoneyKeyValueContainer kvp(current);
+  kvp["provider"] = name();
+  if(m_accountSettings) {
+    kvp.deletePair("kbanking-payee-regexp");
+    kvp.deletePair("kbanking-memo-regexp");
+    kvp.deletePair("kbanking-payee-exceptions");
+    kvp.deletePair("kbanking-txn-download");
+    if(m_accountSettings->m_extractPayeeButton->isChecked()
+    && !m_accountSettings->m_payeeRegExpEdit->text().isEmpty()
+    && !m_accountSettings->m_memoRegExpEdit->text().isEmpty()) {
+      kvp["kbanking-payee-regexp"] = m_accountSettings->m_payeeRegExpEdit->text();
+      kvp["kbanking-memo-regexp"] = m_accountSettings->m_memoRegExpEdit->text();
+      kvp["kbanking-payee-exceptions"] = m_accountSettings->m_payeeExceptions->items().join(";");
+    } else if(m_accountSettings->m_extractPayeeButton->isChecked()) {
+      KMessageBox::information(0, i18n("You selected to extract the payee from the memo field but did not supply a regular expression for payee and memo extraction. The option will not be activated."), i18n("Missing information"));
+    }
+    if(!m_accountSettings->m_transactionDownload->isChecked())
+      kvp["kbanking-txn-download"] = "no";
+  }
+  return kvp;
+}
 
 void KBankingPlugin::createJobView(void)
 {
@@ -123,7 +195,6 @@ void KBankingPlugin::createJobView(void)
   QWidget* w = new KBJobView(m_kbanking, view, "JobView");
   viewInterface()->addWidget(view, w);
   connect(viewInterface(), SIGNAL(viewStateChanged(bool)), frm, SLOT(setEnabled(bool)));
-  connect(viewInterface(), SIGNAL(accountSelected(const MyMoneyAccount&)), this, SLOT(slotAccountSelected(const MyMoneyAccount&)));
 }
 
 
@@ -132,8 +203,6 @@ void KBankingPlugin::createActions(void)
 {
   new KAction(i18n("Configure Aq&Banking..."), "configure", 0, this, SLOT(slotSettings()), actionCollection(), "settings_aqbanking");
   new KAction(i18n("AqBanking importer..."), "", 0, this, SLOT(slotImport()), actionCollection(), "file_import_aqbanking");
-  new KAction(i18n("Map to AqBanking account..."), "news_subscribe", 0, this, SLOT(slotAccountOnlineMap()), actionCollection(), "account_map_aqbanking");
-  new KAction(i18n("Online update using AqBanking..."), "reload", 0, this, SLOT(slotAccountOnlineUpdate()), actionCollection(), "account_update_aqbanking");
 
   connect(viewInterface(), SIGNAL(viewStateChanged(bool)), action("file_import_aqbanking"), SLOT(setEnabled(bool)));
 }
@@ -152,9 +221,10 @@ void KBankingPlugin::slotSettings(void)
 
 
 
-void KBankingPlugin::slotAccountOnlineMap(void)
+bool KBankingPlugin::mapAccount(const MyMoneyAccount& acc, MyMoneyKeyValueContainer& settings)
 {
-  if (!m_account.id().isEmpty()) {
+  bool rc = false;
+  if (!acc.id().isEmpty()) {
     MyMoneyFile* file = MyMoneyFile::instance();
 
     QString bankId;
@@ -162,7 +232,7 @@ void KBankingPlugin::slotAccountOnlineMap(void)
     // extract some information about the bank. if we have a sortcode
     // (BLZ) we display it, otherwise the name is enough.
     try {
-      const MyMoneyInstitution &bank = file->institution(m_account.institutionId());
+      const MyMoneyInstitution &bank = file->institution(acc.institutionId());
       bankId = bank.name();
       if(!bank.sortcode().isEmpty())
         bankId = bank.sortcode();
@@ -173,7 +243,7 @@ void KBankingPlugin::slotAccountOnlineMap(void)
 
     // extract account information. if we have an account number
     // we show it, otherwise the name will be displayed
-    accountId = m_account.number();
+    accountId = acc.number();
     if(accountId.isEmpty())
       accountId = m_account.name();
 
@@ -181,26 +251,78 @@ void KBankingPlugin::slotAccountOnlineMap(void)
     // true, when the user mapped the account or false, if he
     // decided to quit the dialog. So not really a great thing
     // to present some more information.
-    m_kbanking->askMapAccount(m_account.id(),
+    m_kbanking->askMapAccount(acc.id(),
                               bankId.utf8(),
                               accountId.utf8());
+
+    // at this point, the account should be mapped
+    // so we search it and setup the account reference in the KMyMoney object
+    AB_ACCOUNT* ab_acc;
+    ab_acc = AB_Banking_GetAccountByAlias(m_kbanking->getCInterface(), acc.id());
+    if(ab_acc) {
+      MyMoneyAccount a(acc);
+      setupAccountReference(a, ab_acc);
+      settings = a.onlineBankingSettings();
+      rc = true;
+    }
+  }
+  return rc;
+}
+
+QString KBankingPlugin::stripLeadingZeroes(const QString& s) const
+{
+  QString rc(s);
+  QRegExp exp("(0*)(.*)");
+  if(exp.match(s) != -1) {
+    rc = exp.cap(2);
+  }
+  return rc;
+}
+
+void KBankingPlugin::setupAccountReference(const MyMoneyAccount& acc, AB_ACCOUNT* ab_acc)
+{
+  MyMoneyKeyValueContainer kvp;
+
+  if(ab_acc) {
+    QString accountNumber = stripLeadingZeroes(AB_Account_GetAccountNumber(ab_acc));
+    QString routingNumber = stripLeadingZeroes(AB_Account_GetBankCode(ab_acc));
+
+    QString val = QString("%1-%2").arg(routingNumber, accountNumber);
+    if(val != acc.onlineBankingSettings().value("kbanking-acc-ref")) {
+      MyMoneyKeyValueContainer kvp;
+
+      // make sure to keep our own previous settings
+      const QMap<QCString, QString>& vals = acc.onlineBankingSettings().pairs();
+      QMap<QCString, QString>::const_iterator it_p;
+      for(it_p = vals.begin(); it_p != vals.end(); ++it_p) {
+        if(QString(it_p.key()).startsWith("kbanking-")) {
+          kvp.setValue(it_p.key(), *it_p);
+        }
+      }
+
+      kvp.setValue("kbanking-acc-ref", val);
+      kvp.setValue("provider", name());
+      setAccountOnlineParameters(acc, kvp);
+    }
+  } else {
+    // clear the connection
+    setAccountOnlineParameters(acc, kvp);
   }
 }
 
-
-
-const bool KBankingPlugin::accountIsMapped(const QCString& id)
+const bool KBankingPlugin::accountIsMapped(const MyMoneyAccount& acc)
 {
   AB_ACCOUNT* ab_acc;
-  ab_acc = AB_Banking_GetAccountByAlias(m_kbanking->getCInterface(), id);
+  ab_acc = AB_Banking_GetAccountByAlias(m_kbanking->getCInterface(), acc.id());
   return ab_acc != 0;
 }
 
 
 
-void KBankingPlugin::slotAccountOnlineUpdate(void)
+bool KBankingPlugin::updateAccount(const MyMoneyAccount& acc)
 {
-  if (!m_account.id().isEmpty()) {
+  bool rc = false;
+  if (!acc.id().isEmpty()) {
     AB_ACCOUNT *ba;
     AB_JOB *job;
     int rv;
@@ -210,10 +332,10 @@ void KBankingPlugin::slotAccountOnlineUpdate(void)
 
     /* get AqBanking account */
     ba=AB_Banking_GetAccountByAlias(m_kbanking->getCInterface(),
-				    m_account.id());
+                                    acc.id());
     if (!ba) {
       QMessageBox::critical(0,
-			    i18n("Account Not Mapped"),
+                            i18n("Account Not Mapped"),
                             i18n("<qt>"
                                  "<p>"
                                  "The given application account "
@@ -222,79 +344,104 @@ void KBankingPlugin::slotAccountOnlineUpdate(void)
                                  "</p>"
                                  "</qt>"
                             ),
-			    QMessageBox::Ok,QMessageBox::NoButton);
-      return;
+                            QMessageBox::Ok,QMessageBox::NoButton);
+      // clear the connection between the KMyMoney account
+      // and the AqBanking equivalent
+      setupAccountReference(acc, 0);
+      return rc;
     }
+    setupAccountReference(acc, ba);
 
-    /* create getTransactions job */
-    job=AB_JobGetTransactions_new(ba);
-    rv=AB_Job_CheckAvailability(job, 0);
-    if (rv) {
-      DBG_ERROR(0, "Job \"GetTransactions\" is not available (%d)", rv);
-      QMessageBox::critical(0,
-                            i18n("Job not Available"),
-                            i18n("<qt>"
-                                 "The update job is not supported by the "
-                                 "bank/account/backend.\n"
-                                 "</qt>"),
-                            i18n("Dismiss"), QString::null);
-      AB_Job_free(job);
-      return;
-    }
-
-    days=AB_JobGetTransactions_GetMaxStoreDays(job);
-    if (days>0) {
-      GWEN_TIME *ti1;
-      GWEN_TIME *ti2;
-
-      ti1=GWEN_CurrentTime();
-      ti2=GWEN_Time_fromSeconds(GWEN_Time_Seconds(ti1)-(60*60*24*days));
-      GWEN_Time_free(ti1);
-      ti1=ti2;
-
-      if (GWEN_Time_GetBrokenDownDate(ti1, &day, &month, &year)) {
-	DBG_ERROR(0, "Bad date");
-	qd=QDate();
+    if(acc.onlineBankingSettings().value("kbanking-txn-download") != "no") {
+      /* create getTransactions job */
+      job = AB_JobGetTransactions_new(ba);
+      rv = AB_Job_CheckAvailability(job, 0);
+      if (rv) {
+        DBG_ERROR(0, "Job \"GetTransactions\" is not available (%d)", rv);
+        QMessageBox::critical(0,
+                              i18n("Job not Available"),
+                              i18n("<qt>"
+                                  "The update job is not supported by the "
+                                  "bank/account/backend.\n"
+                                  "</qt>"),
+                              i18n("Dismiss"), QString::null);
+        AB_Job_free(job);
+        return rc;
       }
-      else
-	qd=QDate(year, month+1, day);
-      GWEN_Time_free(ti1);
-    }
 
-    // get last statement request date from application account object
-    // and start from the next day if the date is valid
-    QDate lastUpdate = QDate::fromString(m_account.value("lastImportedTransactionDate"), Qt::ISODate);
-    if(lastUpdate.isValid())
-      lastUpdate = lastUpdate.addDays(1);
+      days = AB_JobGetTransactions_GetMaxStoreDays(job);
+      if (days > 0) {
+        GWEN_TIME *ti1;
+        GWEN_TIME *ti2;
 
-    QBPickStartDate psd(m_kbanking, qd, lastUpdate, 
-                        lastUpdate.isValid() ? 2 : 3, 0,
-			"PickStartDate", true);
-    if (psd.exec() != QDialog::Accepted) {
+        ti1=GWEN_CurrentTime();
+        ti2=GWEN_Time_fromSeconds(GWEN_Time_Seconds(ti1)-(60*60*24*days));
+        GWEN_Time_free(ti1);
+        ti1=ti2;
+
+        if (GWEN_Time_GetBrokenDownDate(ti1, &day, &month, &year)) {
+          DBG_ERROR(0, "Bad date");
+          qd=QDate();
+        } else
+          qd=QDate(year, month+1, day);
+        GWEN_Time_free(ti1);
+      }
+
+      // get last statement request date from application account object
+      // and start from the next day if the date is valid
+      QDate lastUpdate = QDate::fromString(acc.value("lastImportedTransactionDate"), Qt::ISODate);
+      if(lastUpdate.isValid())
+        lastUpdate = lastUpdate.addDays(-3);
+
+      KBPickStartDate psd(m_kbanking, qd, lastUpdate,
+                          lastUpdate.isValid() ? 2 : 3, 0,
+                          "PickStartDate", true);
+      if (psd.exec() != QDialog::Accepted) {
+        AB_Job_free(job);
+        return rc;
+      }
+
+      qd=psd.date();
+      if (qd.isValid()) {
+        GWEN_TIME *ti1;
+
+        ti1=GWEN_Time_new(qd.year(), qd.month()-1, qd.day(), 0, 0, 0, 0);
+        AB_JobGetTransactions_SetFromTime(job, ti1);
+        GWEN_Time_free(ti1);
+      }
+
+      rv=m_kbanking->enqueueJob(job);
       AB_Job_free(job);
-      return;
+      if (rv) {
+        DBG_ERROR(0, "Error %d", rv);
+        QMessageBox::critical(0,
+                              i18n("Error"),
+                              i18n("<qt>"
+                                  "Could not enqueue the job.\n"
+                                  "</qt>"),
+                              i18n("Dismiss"), QString::null);
+        return rc;
+      }
     }
 
-    qd=psd.getDate();
-    if (qd.isValid()) {
-      GWEN_TIME *ti1;
+    /* create getBalance job */
+    job = AB_JobGetBalance_new(ba);
+    rv = AB_Job_CheckAvailability(job, 0);
+    if(!rv)
+      rv = m_kbanking->enqueueJob(job);
+    else
+      rv = 0;
 
-      ti1=GWEN_Time_new(qd.year(), qd.month()-1, qd.day(), 0, 0, 0, 0);
-      AB_JobGetTransactions_SetFromTime(job, ti1);
-      GWEN_Time_free(ti1);
-    }
-
-    rv=m_kbanking->enqueueJob(job);
+    AB_Job_free(job);
     if (rv) {
-      AB_Job_free(job);
       DBG_ERROR(0, "Error %d", rv);
       QMessageBox::critical(0,
                             i18n("Error"),
                             i18n("<qt>"
-                                 "Could not enqueue the job.\n"
-                                 "</qt>"),
+                                "Could not enqueue the job.\n"
+                                "</qt>"),
                             i18n("Dismiss"), QString::null);
-      return;
+      return rc;
     }
 
     // ask if the user want's to execute this job right away or spool it
@@ -315,8 +462,6 @@ void KBankingPlugin::slotAccountOnlineUpdate(void)
     if(KMessageBox::questionYesNo(0,
         i18n("Do you want to execute or queue this job in the outbox?"),
         i18n("Execution"), executeButton, queueButton) == KMessageBox::Yes) {
-      AB_JOB_LIST2* jobList = AB_Job_List2_new();
-      AB_Job_List2_PushBack(jobList, job);
 
       AB_IMEXPORTER_CONTEXT *ctx;
 
@@ -331,43 +476,10 @@ void KBankingPlugin::slotAccountOnlineUpdate(void)
 
       // let application emit signals to inform views
       m_kbanking->accountsUpdated();
-
-      AB_Job_List2_free(jobList);
-    }
-    AB_Job_free(job);
-  }
-}
-
-
-
-void KBankingPlugin::slotAccountSelected(const MyMoneyAccount& acc)
-{
-  MyMoneyInstitution institution;
-  bool state = false;
-  m_account = acc;
-
-  action("account_map_aqbanking")->setEnabled(false);
-  action("account_update_aqbanking")->setEnabled(false);
-
-  if (!MyMoneyFile::instance()->isStandardAccount(acc.id())) {
-    switch(m_account.accountGroup()) {
-      case MyMoneyAccount::Asset:
-      case MyMoneyAccount::Liability:
-        state = true;
-        break;
-
-      default:
-        break;
-    }
-  }
-
-  if (state == true) {
-    if (accountIsMapped(acc.id())) {
-      action("account_update_aqbanking")->setEnabled(true);
-    } else {
-      action("account_map_aqbanking")->setEnabled(true);
     }
+    rc = true;
   }
+  return rc;
 }
 
 
@@ -380,14 +492,20 @@ void KBankingPlugin::slotImport(void)
 
 
 
-bool KBankingPlugin::importStatement(MyMoneyStatement& s)
+bool KBankingPlugin::importStatement(const MyMoneyStatement& s)
 {
   return statementInterface()->import(s);
 }
 
+const MyMoneyAccount& KBankingPlugin::account(const QString& key, const QString& value) const
+{
+  return statementInterface()->account(key, value);
+}
 
-
-
+void KBankingPlugin::setAccountOnlineParameters(const MyMoneyAccount& acc, const MyMoneyKeyValueContainer& kvps) const
+{
+  return statementInterface()->setAccountOnlineParameters(acc, kvps);
+}
 
 
 KMyMoneyBanking::KMyMoneyBanking(KBankingPlugin* parent, const char* appname, const char* fname)
@@ -416,7 +534,7 @@ const AB_ACCOUNT_STATUS* KMyMoneyBanking::_getAccountStatus(AB_IMEXPORTER_ACCOUN
       ti=AB_AccountStatus_GetTime(ast);
 
       if (!tiBest) {
-	best=ast;
+        best=ast;
       }
       else {
         if (ti) {
@@ -437,8 +555,9 @@ const AB_ACCOUNT_STATUS* KMyMoneyBanking::_getAccountStatus(AB_IMEXPORTER_ACCOUN
 
 
 
-void KMyMoneyBanking::_xaToStatement(const AB_TRANSACTION *t,
-                                     MyMoneyStatement &ks)
+void KMyMoneyBanking::_xaToStatement(MyMoneyStatement &ks,
+                                     const MyMoneyAccount& acc,
+                                     const AB_TRANSACTION *t)
 {
   const GWEN_STRINGLIST *sl;
   QString s;
@@ -447,6 +566,7 @@ void KMyMoneyBanking::_xaToStatement(const AB_TRANSACTION *t,
   const GWEN_TIME *ti;
   const GWEN_TIME *startTime=0;
   MyMoneyStatement::Transaction kt;
+  unsigned long h;
 
   kt.m_fees = MyMoneyMoney();
 
@@ -469,6 +589,7 @@ void KMyMoneyBanking::_xaToStatement(const AB_TRANSACTION *t,
     }
   }
   kt.m_strPayee=s;
+  h = MyMoneyTransaction::hash(s.simplifyWhiteSpace());
 
   // memo
   s.truncate(0);
@@ -480,13 +601,46 @@ void KMyMoneyBanking::_xaToStatement(const AB_TRANSACTION *t,
     while (se) {
       p = GWEN_StringListEntry_Data(se);
       assert(p);
-      if (!s.isEmpty())
-        s += " ";
       s += QString::fromUtf8(p);
       se = GWEN_StringListEntry_Next(se);
     } // while
   }
   kt.m_strMemo = s;
+  h = MyMoneyTransaction::hash(s.simplifyWhiteSpace(), h);
+
+  // see, if we need to extract the payee from the memo field
+  const MyMoneyKeyValueContainer& kvp = acc.onlineBankingSettings();
+  QString rePayee = kvp.value("kbanking-payee-regexp");
+  if(!rePayee.isEmpty() && kt.m_strPayee.isEmpty()) {
+    QString reMemo = kvp.value("kbanking-memo-regexp");
+    QStringList exceptions = QStringList::split(";", kvp.value("kbanking-payee-exceptions"));
+
+    bool needExtract = true;
+    QStringList::const_iterator it_s;
+    for(it_s = exceptions.begin(); needExtract && it_s != exceptions.end(); ++it_s) {
+      QRegExp exp(*it_s, false);
+      if(exp.search(kt.m_strMemo) != -1) {
+        needExtract = false;
+      }
+    }
+    if(needExtract) {
+      QRegExp expPayee(rePayee, false);
+      QRegExp expMemo(reMemo, false);
+      if(expPayee.search(kt.m_strMemo) != -1) {
+        kt.m_strPayee = expPayee.cap(1);
+        if(expMemo.search(kt.m_strMemo) != -1) {
+          kt.m_strMemo = expMemo.cap(1);
+        }
+      }
+    }
+  }
+
+  // massage whitespaces a bit:
+  // - remove leading blanks
+  // - remove trailing blanks
+  // - reduce multiple blanks to one
+  kt.m_strMemo = kt.m_strMemo.simplifyWhiteSpace();
+  kt.m_strPayee = kt.m_strPayee.simplifyWhiteSpace();
 
   // date
   ti=AB_Transaction_GetDate(t);
@@ -502,8 +656,9 @@ void KMyMoneyBanking::_xaToStatement(const AB_TRANSACTION *t,
         startTime=ti;
     }
 
-    if (!GWEN_Time_GetBrokenDownDate(ti, &day, &month, &year))
+    if (!GWEN_Time_GetBrokenDownDate(ti, &day, &month, &year)) {
       kt.m_datePosted=QDate(year, month+1, day);
+    }
   } else {
     DBG_WARN(0, "No date for transaction");
   }
@@ -514,19 +669,20 @@ void KMyMoneyBanking::_xaToStatement(const AB_TRANSACTION *t,
     if (ks.m_strCurrency.isEmpty()) {
       p=AB_Value_GetCurrency(val);
       if (p)
-	ks.m_strCurrency=p;
+        ks.m_strCurrency=p;
     }
     else {
       p=AB_Value_GetCurrency(val);
       if (p)
-	s=p;
+        s=p;
       if (ks.m_strCurrency.lower()!=s.lower()) {
-	// TODO: handle currency difference
-	DBG_ERROR(0, "Mixed currencies currently not allowed");
+        // TODO: handle currency difference
+        DBG_ERROR(0, "Mixed currencies currently not allowed");
       }
     }
 
     kt.m_amount=MyMoneyMoney(AB_Value_GetValueAsDouble(val));
+    h = MyMoneyTransaction::hash(kt.m_amount.toString(), h);
   }
   else {
     DBG_WARN(0, "No value for transaction");
@@ -553,20 +709,39 @@ void KMyMoneyBanking::_xaToStatement(const AB_TRANSACTION *t,
     DBG_WARN(0, "No date in current transaction");
   }
 
+  // make hash value unique in case we don't have one already
+  if(kt.m_strBankID.isEmpty()) {
+    QString hashBase;
+    hashBase.sprintf("%s-%07lx", kt.m_datePosted.toString(Qt::ISODate).data(), h);
+    int idx = 1;
+    QString hash;
+    for(;;) {
+      hash = QString("%1-%2").arg(hashBase).arg(idx);
+      QMap<QString, bool>::const_iterator it;
+      it = m_hashMap.find(hash);
+      if(it == m_hashMap.end()) {
+        m_hashMap[hash] = true;
+        break;
+      }
+      ++idx;
+    }
+    kt.m_strBankID = QString("%1-%2").arg(acc.id()).arg(hash);
+  }
+
   // store transaction
-  DBG_INFO(0, "Adding transaction");
   ks.m_listTransactions+=kt;
 }
 
 
 
 bool KMyMoneyBanking::importAccountInfo(AB_IMEXPORTER_ACCOUNTINFO *ai,
-					uint32_t /*flags*/)
+                                        uint32_t /*flags*/)
 {
   QString s;
   const char *p;
   const AB_TRANSACTION *t;
   MyMoneyStatement ks;
+  MyMoneyAccount kacc;
   const AB_ACCOUNT_STATUS *ast;
   const AB_VALUE *val;
   const GWEN_TIME *ti;
@@ -575,8 +750,17 @@ bool KMyMoneyBanking::importAccountInfo(AB_IMEXPORTER_ACCOUNTINFO *ai,
 
   // account number
   p=AB_ImExporterAccountInfo_GetAccountNumber(ai);
-  if (p)
-    ks.m_strAccountNumber=p;
+  if (p) {
+    ks.m_strAccountNumber = m_parent->stripLeadingZeroes(p);
+  }
+
+  p=AB_ImExporterAccountInfo_GetBankCode(ai);
+  if(p) {
+    ks.m_strRoutingNumber = m_parent->stripLeadingZeroes(p);
+  }
+
+  kacc = m_parent->account("kbanking-acc-ref", QString("%1-%2").arg(ks.m_strRoutingNumber, ks.m_strAccountNumber));
+  ks.m_accountId = kacc.id();
 
   // account name
   p=AB_ImExporterAccountInfo_GetAccountName(ai);
@@ -585,26 +769,26 @@ bool KMyMoneyBanking::importAccountInfo(AB_IMEXPORTER_ACCOUNTINFO *ai,
 
   // account type
   switch(AB_ImExporterAccountInfo_GetType(ai)) {
-  case AB_AccountType_Bank:
-    ks.m_eType=MyMoneyStatement::etSavings;
-    break;
-  case AB_AccountType_CreditCard:
-    ks.m_eType=MyMoneyStatement::etCreditCard;
-    break;
-  case AB_AccountType_Checking:
-    ks.m_eType=MyMoneyStatement::etCheckings;
-    break;
-  case AB_AccountType_Savings:
-    ks.m_eType=MyMoneyStatement::etSavings;
-    break;
-  case AB_AccountType_Investment:
-    ks.m_eType=MyMoneyStatement::etInvestment;
-    break;
-  case AB_AccountType_Cash:
-    ks.m_eType=MyMoneyStatement::etNone;
-    break;
-  default:
-    ks.m_eType=MyMoneyStatement::etNone;
+    case AB_AccountType_Bank:
+      ks.m_eType=MyMoneyStatement::etSavings;
+      break;
+    case AB_AccountType_CreditCard:
+      ks.m_eType=MyMoneyStatement::etCreditCard;
+      break;
+    case AB_AccountType_Checking:
+      ks.m_eType=MyMoneyStatement::etCheckings;
+      break;
+    case AB_AccountType_Savings:
+      ks.m_eType=MyMoneyStatement::etSavings;
+      break;
+    case AB_AccountType_Investment:
+      ks.m_eType=MyMoneyStatement::etInvestment;
+      break;
+    case AB_AccountType_Cash:
+      ks.m_eType=MyMoneyStatement::etNone;
+      break;
+    default:
+      ks.m_eType=MyMoneyStatement::etNone;
   }
 
   // account status
@@ -612,19 +796,19 @@ bool KMyMoneyBanking::importAccountInfo(AB_IMEXPORTER_ACCOUNTINFO *ai,
   if (ast) {
     const AB_BALANCE *bal;
 
-    bal=AB_AccountStatus_GetBookedBalance(ast);
+    bal = AB_AccountStatus_GetBookedBalance(ast);
     if (!bal)
-      bal=AB_AccountStatus_GetNotedBalance(ast);
+      bal = AB_AccountStatus_GetNotedBalance(ast);
     if (bal) {
-      val=AB_Balance_GetValue(bal);
+      val = AB_Balance_GetValue(bal);
       if (val) {
-	DBG_INFO(0, "Importing balance");
-        ks.m_closingBalance=MyMoneyMoney(AB_Value_GetValueAsDouble(val));
-        p=AB_Value_GetCurrency(val);
+        DBG_INFO(0, "Importing balance");
+        ks.m_closingBalance = MyMoneyMoney(AB_Value_GetValueAsDouble(val));
+        p = AB_Value_GetCurrency(val);
         if (p)
-          ks.m_strCurrency=p;
+          ks.m_strCurrency = p;
       }
-      ti=AB_Balance_GetTime(bal);
+      ti = AB_Balance_GetTime(bal);
       if (ti) {
         int year, month, day;
 
@@ -643,10 +827,13 @@ bool KMyMoneyBanking::importAccountInfo(AB_IMEXPORTER_ACCOUNTINFO *ai,
     DBG_WARN(0, "No account status");
   }
 
+  // clear hash map
+  m_hashMap.clear();
+
   // get all transactions
   t=AB_ImExporterAccountInfo_GetFirstTransaction(ai);
   while(t) {
-    _xaToStatement(t, ks);
+    _xaToStatement(ks, kacc, t);
     t=AB_ImExporterAccountInfo_GetNextTransaction(ai);
   }
 
